import sys
import click
import requests

from pathlib import Path
from requests.models import ChunkedEncodingError, ConnectionError
from requests.exceptions import SSLError
from tqdm import tqdm

DOWNLOAD_FOLDER = Path('.')
"""pathlib.Path: Points to the target directory of downloads."""


def downloader(url: str, resume_byte_pos: int = None, file_name: str = None):
    """Download url in ``URLS[position]`` to disk with possible resumption.

    Parameters
    ----------
    url: str
        url to be downloaded.
    resume_byte_pos: int
        Position of byte from where to resume the download

    """
    # Get size of file
    try:
        r = requests.head(url)
    except ConnectionError:
        click.echo('Connection error! Check your connection')
        sys.exit(1)
    file_size = int(r.headers.get('content-length', 0))

    # Append information to resume download at specific byte position
    # to header
    resume_header = ({'Range': f'bytes={resume_byte_pos}-'}
                     if resume_byte_pos else None)

    # Establish connection
    try:
        r = requests.get(url, stream=True, headers=resume_header)
    except ConnectionError:
        click.echo('Connection error! Check your connection')
        sys.exit(1)
    except SSLError as e:
        click.echo(f'SSLError! {e}')
        sys.exit(1)

    # Set configuration
    block_size = 1024
    initial_pos = resume_byte_pos if resume_byte_pos else 0
    mode = 'ab' if resume_byte_pos else 'wb'
    if file_name is None:
        file = DOWNLOAD_FOLDER / '-'.join(url.split('/')[-3:])
    else:
        file = DOWNLOAD_FOLDER / file_name

    with open(file, mode) as f:
        with tqdm(total=file_size, unit='B',
                  unit_scale=True, unit_divisor=1024,
                  desc=file.name, initial=initial_pos,
                  ascii=True, miniters=1, dynamic_ncols=True) as pbar:
            try:
                for chunk in r.iter_content(32 * block_size):
                    f.write(chunk)
                    pbar.update(len(chunk))
            except ChunkedEncodingError:
                click.echo('Connection Broken! Chunked Encoding Error')
                sys.exit(1)
            except ConnectionError:
                click.echo('Connection Error! Check your connection')
                sys.exit(1)
            except SSLError as e:
                click.echo(f'SSLError! {e}')
                sys.exit(1)

def download_file(url: str, file_name: str = None) -> None:
    """Execute the correct download operation.

    Depending on the size of the file online and offline, resume the
    download if the file offline is smaller than online.

    Parameters
    ----------
    url: str
        url to be downloaded.

    """
    # Establish connection to header of file
    try:
        r = requests.head(url)
    except ConnectionError:
        click.echo('Connection Error! Check your connection')
        sys.exit(1)
    except SSLError as e:
        click.echo(f'SSLError! {e}')
        sys.exit(1)

    # Get filesize of online and offline file
    file_size_online = int(r.headers.get('content-length', 0))
    if file_name is None:
        file = DOWNLOAD_FOLDER / '-'.join(url.split('/')[-3:])
    else:
        file = DOWNLOAD_FOLDER / file_name

    if file.exists():
        file_size_offline = file.stat().st_size

        if file_size_online != file_size_offline:
            click.echo(f'File {file} is incomplete. Resume download.')
            downloader(url, file_size_offline, file)
        else:
            click.echo(f'File {file} is complete. Skip download.')
            pass
    else:
        click.echo(f'File {file} does not exist. Start download.')
        downloader(url, file_name=file)


import sys
import click
import requests
import json
from anid.downloader import download_file 

@click.command()
@click.argument('anime', required=True)
@click.option('-e', '--episodes', required=True, type=int, help='Number of episodes')
@click.option('-s', '--start', required=False, type=int, default=1, help='Number of episodes')
def anid(anime, episodes, start=1):
    headers = {
        "User-Agent": "Mozilla/5.0 (X11; Linux x86_64)"
    }
    base_url = 'https://animefire.net/video/{anime}/{episode}'

    for i in range(start, episodes+1):
        url = base_url.format(anime=anime, episode=i)
        print(url)

        try:
            response = requests.get(url, headers=headers)
        except ConnectionError:
            click.echo('Connection error! Check your connection')
            sys.exit(1)

        if response.ok:
            content = json.loads(response.content)
            if content['response']['status'] == '200':
                download_url = get_url_download(content['data'])
                file_name = f'{anime}-{i}.mp4'
                download_file(download_url, file_name)
                click.echo('')

            else:
                print(content['response']['text'])

        else:
            click.echo(response.status_code)
            click.echo(response.content)
        
def get_url_download(data):
    for src in data:
        if src['label'] == '720p':
            click.echo('Resolution 720p found!')
            return src['src']
    for src in data:
        if src['label'] == '360p':
            click.echo('Resolution 360p found')
            return src['src']

    click.secho(f'Resolutions 720p or 360p not found!', bold=True)
    click.echo(f'Resolutions avaliable:')
    for i in data:
        click.echo(i['label'])
    sys.exit(1)

if __name__ == '__main__':
    anid()


import os
import shutil

import setuptools

currentDir = os.getcwd()
distDir = f'{currentDir}/dist'
buildDir = f'{currentDir}/build'

if os.path.isdir(distDir):
    shutil.rmtree(distDir)

if os.path.isdir(buildDir):
    shutil.rmtree(buildDir)

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="anid",
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': ['anid = anid.anid:anid'],
    },
    version="0.2.1",
    description="Anime Downloader",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="exebixel",
    author_email="ezequielnat7@gmail.com",
    url="https://github.com/exebixel/anid",
    install_requires=["requests", "click", "tqdm"],
    python_requires=">=3.6",
    classifiers=[
        "Operating System :: OS Independent",
        "Environment :: Console",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    zip_safe=False,
)
